@@ -146,6 +146,26 @@ describe ParallelTests::Test::Runner do
 
         expect(valid_combinations).to include(actual)
       end
+
+      it "groups by size and use specified number of isolation groups" do
+        skip if RUBY_PLATFORM == "java"
+        expect(ParallelTests::Test::Runner).to receive(:runtimes).
+          and_return({"aaa1" => 1, "aaa2" => 3, "aaa3" => 2, "bbb" => 3, "ccc" => 1, "ddd" => 2})
+        result = call(["aaa1", "aaa2", "aaa3", "bbb", "ccc", "ddd", "eee"], 4, isolate_count: 2, single_process: [/^aaa/], group_by: :runtime)
+
+        isolated_1, isolated_2, *groups = result
+        expect(isolated_1).to eq(["aaa2"])
+        expect(isolated_2).to eq(["aaa1", "aaa3"])
+        actual = groups.map(&:to_set).to_set
+
+        # both eee and ccs are the same size, so either can be in either group
+        valid_combinations = [
+          [["bbb", "eee"], ["ccc", "ddd"]].map(&:to_set).to_set,
+          [["bbb", "ccc"], ["eee", "ddd"]].map(&:to_set).to_set
+        ]
+
+        expect(valid_combinations).to include(actual)
+      end
     end
   end
 
